"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = __importStar(require("underscore"));
const JSON5 = __importStar(require("json5"));
const sequelize_1 = __importDefault(require("sequelize"));
const ck_lib_1 = require("@ckstack/ck-lib");
const index_1 = require("../index");
const MyValues_1 = require("./MyValues");
class DatabaseService {
    static init() {
        ck_lib_1.CkLogger.infoLog2('DatabaseService.init.', MyValues_1.PROC_LHD.IP_LHD_HTTP);
    }
    static getDataOneById(reqIpAndLhdAndHttpInfo, TCreator, id, attributes, transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            const lhd = reqIpAndLhdAndHttpInfo || ck_lib_1.CkValue.getDefaultLogKey(true);
            if (!id) {
                ck_lib_1.CkLogger.errLog2(`${lhd} no id for getDataOneById. id [${id}]`);
                return null;
            }
            let retData = null;
            try {
                const findOpts = {
                    where: { id },
                    logging: (sql, timing) => {
                        ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                        ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `Tb${TCreator.name}.findOne - where id [${id}]`);
                    },
                };
                if (attributes && attributes.length > 0) {
                    findOpts.attributes = attributes;
                }
                findOpts.transaction = transaction || undefined;
                retData = yield TCreator.findOne(findOpts);
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog2(`can't find data in Tb${TCreator.name}. DB error [${err.toString()}], [${err.original}], id [${id}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            return retData;
        });
    }
    static getDatasByIds(reqIpAndLhdAndHttpInfo, TCreator, ids, attributes, transaction, order) {
        return __awaiter(this, void 0, void 0, function* () {
            const lhd = reqIpAndLhdAndHttpInfo || ck_lib_1.CkValue.getDefaultLogKey(true);
            if (_.size(ids) <= 0) {
                ck_lib_1.CkLogger.errLog2(`no id for getDatasByIds. ids [${JSON5.stringify(ids)}]`, lhd);
                return [];
            }
            if (ids.length === 1) {
                const oneRet = yield DatabaseService.getDataOneById(lhd, TCreator, ids[0], attributes, transaction);
                if (_.isString(oneRet)) {
                    return oneRet;
                }
                if (oneRet === null) {
                    return [];
                }
                return [oneRet];
            }
            let retData = [];
            try {
                const findOpts = {
                    where: { id: { $in: ids } },
                    logging: (sql, timing) => {
                        ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                        ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `Tb${TCreator.name}.findAll - where ids [${JSON5.stringify(ids)}]`);
                    },
                };
                if (attributes && attributes.length > 0) {
                    findOpts.attributes = attributes;
                }
                findOpts.transaction = transaction || undefined;
                findOpts.order = order || undefined;
                retData = yield TCreator.findAll(findOpts);
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog2(`can't find data in Tb${TCreator.name}. DB error [${err.toString()}], [${err.original}], ids [${JSON5.stringify(ids)}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            return retData;
        });
    }
    static countDatasByIds(reqIpAndLhdAndHttpInfo, TCreator, ids, transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            const lhd = reqIpAndLhdAndHttpInfo || ck_lib_1.CkValue.getDefaultLogKey(true);
            if (ids.length <= 0) {
                ck_lib_1.CkLogger.errLog2(`no id for countDatasByIds. ids [${JSON5.stringify(ids)}]`, lhd);
                return 0;
            }
            let retData = 0;
            try {
                const findOpts = {
                    where: { id: { $in: ids } },
                    logging: (sql, timing) => {
                        ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                        ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `Tb${TCreator.name}.count - where ids [${JSON5.stringify(ids)}]`);
                    },
                };
                findOpts.transaction = transaction || undefined;
                retData = yield TCreator.count(findOpts);
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog2(`can't count data in Tb${TCreator.name}. DB error [${err.toString()}], [${err.original}], ids [${JSON5.stringify(ids)}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            return retData;
        });
    }
    static getDataOneByRid(reqIpAndLhdAndHttpInfo, TCreator, rid, attributes, transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            const lhd = reqIpAndLhdAndHttpInfo || ck_lib_1.CkValue.getDefaultLogKey(true);
            if (!rid) {
                ck_lib_1.CkLogger.errLog2(`no rid for getDataOneByRid. rid [${rid}]`, lhd);
                return null;
            }
            let retData = null;
            try {
                const findOpts = {
                    where: { rid },
                    logging: (sql, timing) => {
                        ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                        ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `Tb${TCreator.name}.findOne - where rid [${rid}]`);
                    },
                };
                if (attributes && attributes.length > 0) {
                    findOpts.attributes = attributes;
                }
                findOpts.transaction = transaction || undefined;
                retData = yield TCreator.findOne(findOpts);
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog2(`can't find data in Tb${TCreator.name}. DB error [${err.toString()}], [${err.original}], rid [${rid}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            return retData;
        });
    }
    static getDatasByRids(reqIpAndLhdAndHttpInfo, TCreator, rids, attributes, transaction, order) {
        return __awaiter(this, void 0, void 0, function* () {
            const lhd = reqIpAndLhdAndHttpInfo || ck_lib_1.CkValue.getDefaultLogKey(true);
            if (!rids || rids.length <= 0) {
                ck_lib_1.CkLogger.errLog2(`no rid for getDatasByRids. rids [${JSON5.stringify(rids)}]`, lhd);
                return [];
            }
            if (rids.length === 1) {
                const oneRet = yield DatabaseService.getDataOneByRid(lhd, TCreator, rids[0], attributes, transaction);
                if (_.isString(oneRet)) {
                    return oneRet;
                }
                if (oneRet === null) {
                    return [];
                }
                return [oneRet];
            }
            let retData = [];
            try {
                const findOpts = {
                    where: { rid: { $in: rids } },
                    logging: (sql, timing) => {
                        ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                        ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `Tb${TCreator.name}.findAll - where rids [${JSON5.stringify(rids)}]`);
                    },
                };
                if (attributes && attributes.length > 0) {
                    findOpts.attributes = attributes;
                }
                findOpts.transaction = transaction || undefined;
                findOpts.order = order || undefined;
                retData = yield TCreator.findAll(findOpts);
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog2(`can't find data in Tb${TCreator.name}. DB error [${err.toString()}], [${err.original}], rids [${JSON5.stringify(rids)}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            return retData;
        });
    }
    static countDatasByRids(reqIpAndLhdAndHttpInfo, TCreator, rids, transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            const lhd = reqIpAndLhdAndHttpInfo || ck_lib_1.CkValue.getDefaultLogKey(true);
            if (!rids || rids.length <= 0) {
                ck_lib_1.CkLogger.errLog2(`no rid for countDatasByRids. rids [${JSON5.stringify(rids)}]`, lhd);
                return 0;
            }
            let retData = 0;
            try {
                const findOpts = {
                    where: { rid: { $in: rids } },
                    logging: (sql, timing) => {
                        ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                        ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `Tb${TCreator.name}.count - where rids [${JSON5.stringify(rids)}]`);
                    },
                };
                findOpts.transaction = transaction || undefined;
                retData = yield TCreator.count(findOpts);
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog2(`can't count data in Tb${TCreator.name}. DB error [${err.toString()}], [${err.original}], rids [${JSON5.stringify(rids)}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            return retData;
        });
    }
    static getDataOneByFilter(reqIpAndLhdAndHttpInfo, TCreator, filter, attributes, transaction, order) {
        return __awaiter(this, void 0, void 0, function* () {
            const lhd = reqIpAndLhdAndHttpInfo || ck_lib_1.CkValue.getDefaultLogKey(true);
            if (!filter || _.size(filter) <= 0) {
                ck_lib_1.CkLogger.errLog2(`no filter for getDataOneByFilter. filter [${JSON5.stringify(filter)}]`, lhd);
                return null;
            }
            let retData = null;
            try {
                const findOpts = {
                    where: filter,
                    logging: (sql, timing) => {
                        ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                        ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `Tb${TCreator.name}.findOne - where filter [${JSON5.stringify(filter)}]`);
                    },
                };
                if (attributes && attributes.length > 0) {
                    findOpts.attributes = attributes;
                }
                findOpts.transaction = transaction || undefined;
                findOpts.order = order || undefined;
                retData = yield TCreator.findOne(findOpts);
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog(`can't find data in Tb${TCreator.name}. DB error [${err.toString()}], [${err.original}], filter [${JSON5.stringify(filter)}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            return retData;
        });
    }
    static getDatasByFilter(reqIpAndLhdAndHttpInfo, TCreator, filter, attributes, transaction, order, offset, limit) {
        return __awaiter(this, void 0, void 0, function* () {
            const lhd = reqIpAndLhdAndHttpInfo || ck_lib_1.CkValue.getDefaultLogKey(true);
            if ((!filter || _.size(filter) <= 0) && (limit === undefined || limit <= 0)) {
                ck_lib_1.CkLogger.errLog2(`no filter and no limit for getDatasByFilter. filter [${JSON5.stringify(filter)}], limit [${limit}]`, lhd);
                return [];
            }
            let retData = [];
            try {
                const findOpts = {
                    logging: (sql, timing) => {
                        ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                        ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `Tb${TCreator.name}.findOne - where filter [${JSON5.stringify(filter)}]`);
                    },
                };
                if (filter && _.size(filter) > 0) {
                    findOpts.where = filter;
                }
                if (attributes && attributes.length > 0) {
                    findOpts.attributes = attributes;
                }
                findOpts.transaction = transaction || undefined;
                findOpts.order = order || undefined;
                if (offset !== undefined) {
                    findOpts.offset = offset;
                }
                if (limit !== undefined) {
                    findOpts.limit = limit;
                }
                retData = yield TCreator.findAll(findOpts);
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog2(`can't find data in Tb${TCreator.name}. DB error [${err.toString()}], [${err.original}], filter [${JSON5.stringify(filter)}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            return retData;
        });
    }
    static countDatasByFilter(reqIpAndLhdAndHttpInfo, TCreator, filter, transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            const lhd = reqIpAndLhdAndHttpInfo || ck_lib_1.CkValue.getDefaultLogKey(true);
            if (!filter || _.size(filter) <= 0) {
                ck_lib_1.CkLogger.warnLog2(`no filter for countDatasByFilter. filter [${JSON5.stringify(filter)}]`, lhd);
            }
            let retData = 0;
            try {
                const findOpts = {
                    where: filter,
                    logging: (sql, timing) => {
                        ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                        ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `Tb${TCreator.name}.count - where filter [${JSON5.stringify(filter)}]`);
                    },
                };
                findOpts.transaction = transaction || undefined;
                retData = yield TCreator.count(findOpts);
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog2(`can't count data in Tb${TCreator.name}. DB error [${err.toString()}], [${err.original}], filter [${JSON5.stringify(filter)}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            return retData;
        });
    }
    static getDataMapByKeyArray(reqIpAndLhdAndHttpInfo, keys, VCreator, colName, mapKey, cols, toCond, transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            const lhd = reqIpAndLhdAndHttpInfo || ck_lib_1.CkValue.getDefaultLogKey(true);
            let ret = {};
            if (_.size(keys) > 0) {
                try {
                    let where = { [colName]: { $in: keys } };
                    if (toCond) {
                        where = Object.assign(Object.assign({}, toCond), where);
                    }
                    const datas = yield VCreator.findAll({
                        where,
                        attributes: (cols && cols.length > 0 ? cols : undefined),
                        logging: (sql, timing) => {
                            ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                            ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `Tb${VCreator.name}.findAll - ${colName} [${JSON5.stringify(keys)}]`);
                        },
                        transaction: transaction || undefined,
                    });
                    if (_.size(datas) > 0) {
                        ret = ck_lib_1.CkUtils.arrayToMaps(datas, mapKey);
                    }
                }
                catch (err) {
                    ck_lib_1.CkLogger.warnLog2(`can't find Tb${VCreator.name} data. ${colName} [${JSON5.stringify(keys)}] DB err [${err}], [${err.original}]`, lhd);
                }
            }
            return ret;
        });
    }
    static changeListOrder(reqIpAndLhdAndHttpInfo, rdb, TCreator, rawQuery, replacements, srcId, srcListOrder, destListOrder, transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            const lhd = reqIpAndLhdAndHttpInfo || ck_lib_1.CkValue.getDefaultLogKey(true);
            let totalUpdatedCount = 0;
            if (srcListOrder > destListOrder) {
                const _replacements = replacements !== undefined
                    ? Object.assign(Object.assign({}, replacements), { lt: srcListOrder, gte: destListOrder }) : { lt: srcListOrder, gte: destListOrder };
                let _rawQuery = `${rawQuery.replace('<list_order>', 'list_order = list_order + 1')} `;
                _rawQuery += (_rawQuery.toLowerCase().indexOf(' where ') > 0)
                    ? ' AND ( list_order >= :gte AND list_order < :lt ) '
                    : ' WHERE list_order >= :gte AND list_order < :lt ';
                ck_lib_1.CkLogger.debugLog2(`list_order move up raw query [${_rawQuery}], replacements [${JSON5.stringify(_replacements)}]`, lhd);
                try {
                    const destUpdated = yield rdb.query(_rawQuery, {
                        raw: true,
                        model: TCreator,
                        type: sequelize_1.default.QueryTypes.RAW,
                        replacements: _replacements,
                        transaction,
                        logging: (sql, timing) => {
                            ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                            ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `changeOrder(rawQuery) - replacements [${JSON5.stringify(_replacements)}]`);
                        },
                    });
                    ck_lib_1.CkLogger.debugLog2(`move up. dest order. where [${JSON5.stringify(_replacements)}], +1 list order. updated result [${JSON5.stringify(destUpdated)}]`, lhd);
                    if (destUpdated && _.size(destUpdated) >= 0) {
                        ck_lib_1.CkLogger.infoLog2(`move up. dest order updated count [${destUpdated[0].affectedRows}]`, lhd);
                        totalUpdatedCount += Number(destUpdated[0].affectedRows);
                    }
                }
                catch (err) {
                    ck_lib_1.CkLogger.errLog2(`can't change dest list_order. DB error [${err.toString()}], [${err.original}], where [${JSON5.stringify(_replacements)}]`, lhd);
                    throw err;
                }
            }
            else {
                const _replacements = (replacements !== undefined)
                    ? Object.assign(Object.assign({}, replacements), { lte: destListOrder, gt: srcListOrder }) : { lte: destListOrder, gt: srcListOrder };
                let _rawQuery = `${rawQuery.replace('<list_order>', 'list_order = list_order - 1')} `;
                _rawQuery += (_rawQuery.toLowerCase().indexOf(' where ') > 0)
                    ? ' AND ( list_order > :gt AND list_order <= :lte ) '
                    : ' WHERE list_order > :gt AND list_order <= :lte ';
                ck_lib_1.CkLogger.debugLog2(`list_order move down raw query [${_rawQuery}], replacements [${JSON5.stringify(_replacements)}]`, lhd);
                try {
                    const destUpdated = yield rdb.query(_rawQuery, {
                        raw: true,
                        model: TCreator,
                        type: sequelize_1.default.QueryTypes.RAW,
                        replacements: _replacements,
                        transaction,
                        logging: (sql, timing) => {
                            ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                            ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `changeOrder(rawQuery) - replacements [${JSON5.stringify(_replacements)}]`);
                        },
                    });
                    ck_lib_1.CkLogger.debugLog2(`move down. dest order. where [${JSON5.stringify(_replacements)}], -1 list order. updated result [${JSON5.stringify(destUpdated)}]`, lhd);
                    if (destUpdated && _.size(destUpdated) > 1) {
                        ck_lib_1.CkLogger.infoLog2(`move down. dest order updated count [${destUpdated[0].affectedRows}]`, lhd);
                        totalUpdatedCount += Number(destUpdated[0].affectedRows);
                    }
                }
                catch (err) {
                    ck_lib_1.CkLogger.errLog2(`can't change dest list_order. DB error [${err.toString()}], [${err.original}], where [${JSON5.stringify(_replacements)}]`, lhd);
                    throw err;
                }
            }
            try {
                const srcUpdated = yield TCreator.update({ list_order: destListOrder }, {
                    where: { id: srcId },
                    logging: (sql, timing) => {
                        ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                        ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `changeOrder(src list_order) - src srl [${srcId}], set list_order [${destListOrder}]`);
                    },
                    transaction,
                });
                ck_lib_1.CkLogger.debugLog2(`set src order. src srl [${srcId}], set list_order [${destListOrder}]`, lhd);
                ck_lib_1.CkLogger.infoLog2(`change src order updated count [${srcUpdated[0]}], src srl [${srcId}]`, lhd);
                totalUpdatedCount += Number(srcUpdated[0]);
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog2(`can't update src list_order. DB error [${err.toString()}], [${err.original}], where src srl [${srcId}], set list_order [${destListOrder}]`, lhd);
                throw err;
            }
            return totalUpdatedCount;
        });
    }
    static changeListOrderV2(reqIpAndLhdAndHttpInfo, rdb, TCreator, srcRid, destRid, rawQueryPart, replacements, transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            const lhd = reqIpAndLhdAndHttpInfo || ck_lib_1.CkValue.getDefaultLogKey(true);
            let dataMap = {};
            try {
                const tmpData = yield TCreator.findAll({
                    where: {
                        rid: { $in: [srcRid.trim(), destRid.trim()] },
                    },
                    attributes: ['id', 'rid', 'list_order'],
                    offset: 0,
                    limit: 2,
                    logging: (sql, timing) => {
                        ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                        ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `${TCreator.name}.findAll - where rid [${srcRid}, ${destRid}]`);
                    },
                    transaction,
                });
                dataMap = ck_lib_1.CkUtils.arrayToMaps(tmpData, 'rid');
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog2(`can't change order. DB error [${err.toString()}], [${err.original}], srcId [${srcRid}], destId [${destRid}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            let srcData;
            let srcSrl;
            if (dataMap[`k_${srcRid.trim()}`]) {
                srcData = dataMap[`k_${srcRid.trim()}`];
                srcSrl = dataMap[`k_${srcRid.trim()}`].id;
            }
            if (srcData === undefined || srcSrl === undefined) {
                ck_lib_1.CkLogger.errLog2(`not found src data for change list order. srcId [${srcRid}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.wrong_request;
            }
            let destData;
            let destSrl;
            if (dataMap[`k_${destRid.trim()}`]) {
                destData = dataMap[`k_${destRid.trim()}`];
                destSrl = dataMap[`k_${destRid.trim()}`].id;
            }
            if (destData === undefined || destSrl === undefined) {
                ck_lib_1.CkLogger.errLog2(`not found dest data for change list order. destId [${destRid}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.wrong_request;
            }
            if (srcData.list_order === destData.list_order) {
                ck_lib_1.CkLogger.errLog2(`src list_order and dest list_order is same. src data id [${srcData.id}], src data list_order [${srcData.list_order}], dest data id [${destData.id}], dest data list_order [${destData.list_order}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.wrong_request;
            }
            let updatedCount = 0;
            try {
                updatedCount = yield DatabaseService.changeListOrder(lhd, rdb, TCreator, rawQueryPart, replacements, srcData.id, srcData.list_order, destData.list_order, transaction);
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog2("can't change list_order", lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            ck_lib_1.CkLogger.infoLog2(`changed list_order. src srl [${srcData.id}], from [${srcData.list_order}], to [${destData.list_order}], updated count [${updatedCount}]`, lhd);
            return updatedCount;
        });
    }
    static createDataWithListOrder(reqIpAndLhdAndHttpInfo, TCreator, newData, transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            const lhd = reqIpAndLhdAndHttpInfo || ck_lib_1.CkValue.getDefaultLogKey(true);
            let savedData = null;
            try {
                savedData = yield newData.save({
                    logging: (sql, timing) => {
                        ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                        ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `${newData.constructor.name}.save - data [${JSON5.stringify(newData)}]`);
                    },
                    transaction,
                });
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog2(`can't create data. DB error [${err.toString()}], [${err.original}], data [${JSON5.stringify(newData)}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            if (!savedData) {
                ck_lib_1.CkLogger.errLog2('not found new created data', lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            ck_lib_1.CkLogger.infoLog2(`saved new DB data(${newData.constructor.name}). id [${savedData.id}]`, lhd);
            try {
                const updatedData = yield TCreator.update({ list_order: savedData.id }, {
                    where: { id: savedData.id },
                    logging: (sql, timing) => {
                        ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                        ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `${TCreator.name}.update - where id [${savedData === null || savedData === void 0 ? void 0 : savedData.id}], set list_order [${savedData === null || savedData === void 0 ? void 0 : savedData.id}]`);
                    },
                    transaction,
                });
                ck_lib_1.CkLogger.infoLog2(`set list_order of new DB data. id [${savedData.id}], list_order [${savedData.id}], updated count [${updatedData[0]}]`, lhd);
                savedData.list_order = savedData.id;
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog2(`can't update list_order. DB error [${err.toString()}], [${err.original}], id [${savedData.id}], list_order [${savedData.id}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            return savedData;
        });
    }
    static createData(reqIpAndLhdAndHttpInfo, newData, transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            const lhd = reqIpAndLhdAndHttpInfo || ck_lib_1.CkValue.getDefaultLogKey(true);
            let savedData = null;
            try {
                savedData = yield newData.save({
                    logging: (sql, timing) => {
                        ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                        ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `${newData.constructor.name}.save - data [${JSON5.stringify(newData)}]`);
                    },
                    transaction,
                });
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog2(`can't create data [2]. DB error [${err.toString()}], [${err.original}], data [${JSON5.stringify(newData)}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            if (!savedData) {
                ck_lib_1.CkLogger.errLog2('not found new created data', lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            return savedData;
        });
    }
    static updateData(reqIpAndLhdAndHttpInfo, TCreate, modData, where, transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            const lhd = reqIpAndLhdAndHttpInfo || ck_lib_1.CkValue.getDefaultLogKey(true);
            let updatedData;
            try {
                updatedData = yield TCreate.update(modData, {
                    where,
                    logging: (sql, timing) => {
                        ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                        ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `${TCreate.name}.update - where [${JSON5.stringify(where)}], modData [${JSON5.stringify(modData)}]`);
                    },
                    transaction,
                });
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog2(`can't update data. DB error [${err.toString()}], [${err.original}], where [${JSON5.stringify(where)}], modData [${JSON5.stringify(modData)}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            return updatedData[0];
        });
    }
    static deleteData(reqIpAndLhdAndHttpInfo, TCreate, where, transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            const lhd = reqIpAndLhdAndHttpInfo || ck_lib_1.CkValue.getDefaultLogKey(true);
            let rmCount = 0;
            try {
                rmCount = yield TCreate.destroy({
                    where,
                    logging: (sql, timing) => {
                        ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                        ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `${TCreate.name}.delete - where [${JSON5.stringify(where)}]`);
                    },
                    transaction,
                });
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog2(`can't delete data. DB error [${err.toString()}], [${err.original}], where [${JSON5.stringify(where)}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            return rmCount;
        });
    }
    static incrementData(reqIpAndLhdAndHttpInfo, TCreate, where, cols, by, transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            const lhd = reqIpAndLhdAndHttpInfo || ck_lib_1.CkValue.getDefaultLogKey(true);
            try {
                yield TCreate.increment(cols, {
                    where,
                    by,
                    logging: (sql, timing) => {
                        ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                        ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `${TCreate.name}.increment - where [${JSON5.stringify(where)}], cols [${JSON5.stringify(cols)}], by [${by}]`);
                    },
                    transaction,
                });
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog2(`can't increment data. DB error [${err.toString()}], [${err.original}], where [${JSON5.stringify(where)}], cols [${JSON5.stringify(cols)}], by [${by}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            return undefined;
        });
    }
    static getDataByRawQuery(reqIpAndLhdAndHttpInfo, rawQuery, rdb, replacements, transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            const lhd = reqIpAndLhdAndHttpInfo || ck_lib_1.CkValue.getDefaultLogKey(true);
            const _rdb = _.isNumber(rdb) ? index_1.CkSequelize.getSequelize2(lhd, rdb) : rdb;
            if (!_rdb) {
                ck_lib_1.CkLogger.errLog2(`can't database select by raw query [${rawQuery}], invalid database number [${rdb}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            let result = [];
            try {
                result = yield _rdb.query(rawQuery, {
                    raw: true,
                    type: sequelize_1.default.QueryTypes.SELECT,
                    replacements,
                    transaction,
                    logging: (sql, timing) => {
                        ck_lib_1.CkLogger.debugLog2(`SQL=[${sql}], elapsed [${timing} ms]`, lhd);
                        ck_lib_1.CkLogger.printElapsedSqlLog2(lhd, sql, timing, `raw query [${rawQuery}], replacements [${JSON5.stringify(replacements)}]`);
                    },
                });
            }
            catch (err) {
                ck_lib_1.CkLogger.errLog2(`can't run raw query. DB error [${err.toString()}], [${err.original}], raw query [${rawQuery}], replacements [${JSON5.stringify(replacements)}]`, lhd);
                return ck_lib_1.CkValue.ERROR_TYPE.db_error;
            }
            return result;
        });
    }
    static addMandatoryColumns(columnNames, attributes) {
        let retAttr;
        if (!attributes) {
            return retAttr;
        }
        retAttr = attributes;
        for (const row of columnNames) {
            if (attributes.findIndex((col) => col === row) < 0) {
                retAttr.push(row);
            }
        }
        return retAttr;
    }
}
DatabaseService.LHD = '[ck-lib-sequelize:]';
exports.default = DatabaseService;
